// import the IFC
import netscape.application.*;
import netscape.util.*;

public class ApplicationViewer implements WindowOwner {
  ExternalWindow win;  // a window to show the application in
  Application app;     // the application that will be 'viewed.'

  // The main() method.  Invoked when run from the command line.
  // Create and run an Application with name args[0] in a window with
  // width and height specified by optional args[1] and args[2].
  public static void main(String[] args){ 
    ApplicationViewer appViewer = new ApplicationViewer();
    int width = 600;
    int height = 400;
    if (args.length >= 2) width = Integer.parseInt(args[1]);
    if (args.length >= 3) height = Integer.parseInt(args[2]);
    appViewer.createApplication(args[0], width, height);
    System.exit(0);
  }

  // The next two methods run Applications from within other programs.
  // Create and run an Application with name className in the default window
  public static void runApplication(String className) {
    ApplicationViewer appViewer = new ApplicationViewer();
    appViewer.createApplication(className, 600, 400);
  }  

  // Create and run an Application with name className in a
  // window with the specified width and height
  public static void runApplication(String className, int width, int height) {
    ApplicationViewer appViewer = new ApplicationViewer();
    appViewer.createApplication(className, width, height);
  }

  // This is the method that does all the real work of running an Application.
  // Load and instantiate the Application class, create a window for it,
  // place it in the window, and start it running.
  private void createApplication(String className, int width, int height) {
    // Try to load the Application class
    Class appClass = null;
    try { appClass = Class.forName(className); }
    catch (ClassNotFoundException e) {
      System.err.println(className + " not found.  Exiting.");
      System.exit(0);
    }

    // Try to instantiate the Application
    Object o = null;
    try { o = appClass.newInstance(); }
    catch (InstantiationException e) {
      System.err.println(className + ": could not instantiate.  Exiting.");
      System.exit(0);
    }
    catch(IllegalAccessException e) {
      System.err.println(className + 
			 " is not public or has no public constructor.  " +
			 " Exiting.");
      System.exit(0);
    }

    // Make sure it really is an Application subclass
    if (!(o instanceof Application)) {
      System.err.println(className + " is not an Application.  Exiting.");
      System.exit(0);
    }
    app = (Application) o;

    // Now create a window for the application.  Set the window size, etc.
    win = new ExternalWindow();
    Size s = win.windowSizeForContentSize(width, height);
    win.sizeTo(s.width, s.height);
    win.setTitle(className);
    win.setOwner(this);

    app.setMainRootView(win.rootView()); // Put app in the window
    win.show();                          // Pop the window up
    app.run();                           // Start running the application
  } 
  
  // WindowOwner interface methods
  // This one is called when the user closes the window.
  public void windowDidHide(Window win) { 
    this.win.dispose(); // Get rid of the window.
    app.stopRunning();  // Tell the application to stop.  (Makes run() return).
  }
  
  // other WindowOwner methods are given no-op implementations
  public void windowDidShow(Window win){}
  public boolean windowWillHide(Window win){return true;}
  public void windowDidBecomeMain(Window win){}
  public void windowDidResignMain(Window win){}
  public void windowWillSizeBy(Window win,Size size){}
  public boolean windowWillShow(Window win){return true;}
} 

