import netscape.application.*;
import netscape.util.*;

// A drawing application
public class SuperScribble extends Application{
  SuperScribbleView sview;  
  
  public void init(){
    sview = new SuperScribbleView(mainRootView().localBounds());
    mainRootView().addSubview(sview);
  }
}

// A View to draw in, with 5 buttons to set the
// color and line width.
class SuperScribbleView extends View implements Target{
  int lastx,lasty;
  Button clearButton,thickButton;
  Button red,green,blue;
  ContainerView radioContainer;
  Color currentColor;
  boolean wasOutOfBounds = false;
  boolean thickLines = false;
  
  public SuperScribbleView(Rect r){
    super(r);
    currentColor = Color.blue;
    
    // create the buttons
    red = Button.createRadioButton(0,0,r.width/3,50);
    setupButton(red,"Red",this,"red");
    blue = Button.createRadioButton(r.width/3,0,r.width/3,50);
    setupButton(blue,"Blue",this,"blue");
    blue.setState(true);  // blue is the initial line color
    green = Button.createRadioButton(2*r.width/3,0,r.width/3,50);
    setupButton(green,"Green",this,"green");
    clearButton = new Button(10,10,100,30);
    setupButton(clearButton,"Clear All",this,"clear");
    thickButton = Button.createCheckButton(10,60,100,30);
    setupButton(thickButton,"Thick Lines",this,"thickLines");

    // put the radio buttons in a ContainerView
    radioContainer = new ContainerView(0,r.height-50,r.width,50);
    radioContainer.setTitle("Colors");
    
    radioContainer.addSubview(red);
    radioContainer.addSubview(blue);
    radioContainer.addSubview(green);
    addSubview(radioContainer);
    
    addSubview(clearButton);
    addSubview(thickButton);
  }
  
  // a convenient method to set a button's attributes
  protected void setupButton(Button b,String title,
                             Target target,String command){
    b.setTitle(title);
    b.setCommand(command);
    b.setTarget(target);
  }
  
  public void performCommand(String command,Object data){
    if(command.equals("red")) currentColor = Color.red;
    if(command.equals("blue")) currentColor = Color.blue;
    if(command.equals("green")) currentColor = Color.green;
    if(command.equals("thickLines")) thickLines = thickButton.state();
    if(command.equals("clear")) draw();
  }
  
  public void drawView(Graphics g){ 
    g.setColor(Color.white);
    g.fillRect(localBounds());
  } 
  
  public boolean mouseDown(MouseEvent e){
    wasOutOfBounds = false;  // the MouseEvent is in the view
    lastx = e.x;
    lasty = e.y;
    return true;
  }
  
  public void mouseDragged(MouseEvent e){
    // if the mouse dragged out of the view
    // or over a button, set a flag and return.
    if(!containsPointInVisibleRect(e.x,e.y) || 
       viewForMouse(e.x,e.y) != this){
      wasOutOfBounds = true;
      return; 
    }

    // if the mouse was dragged out of the view
    // or over a button, ignore it
    if(wasOutOfBounds) return;

    // prepare to draw
    Graphics gr = createGraphics();
    gr.setColor(currentColor);
    int dx = lastx - e.x;
    int dy = lasty - e.x;

    // draw the line
    gr.drawLine(lastx,lasty,e.x,e.y);
    
    // if the lines are "thick", draw four lines 
    // around the center line  
    if(thickLines){
      gr.drawLine(lastx-1,lasty,e.x-1,e.y);
      gr.drawLine(lastx+1,lasty,e.x+1,e.y);
      gr.drawLine(lastx,lasty-1,e.x,e.y-1);
      gr.drawLine(lastx,lasty+1,e.x,e.y+1);
    }

    gr.dispose();  // dispose of the Graphics object

    // update the mouse position
    lastx = e.x;
    lasty = e.y;
  }
}
