import netscape.application.*;
import netscape.util.*;

// an example that uses a Bitmap for off-screen drawing
public class Buffered extends Application{
  Roller roller;
  OffScreenRoller osRoller;
  
  public void init(){
    Rect rect = mainRootView().localBounds();
    roller = new Roller(0,rect.height/2,rect.width,rect.height/2);
    osRoller = new OffScreenRoller(0,0,rect.width,rect.height/2);
    mainRootView().addSubview(roller);
    mainRootView().addSubview(osRoller);
    roller.draw();
    osRoller.draw();
    roller.start();   // start the animation
    osRoller.start(); // start the animation
  }
  
  public void cleanup(){
    roller.stop();
    osRoller.stop();
  }
}

// a class that does some animation using the View's
// built-in buffered drawing capabilities
class Roller extends View implements Runnable{
  Thread thread;
  int x;
  int angle;
  Rect rect;
  int diam = 30;
 
  public Roller(int x,int y,int w,int h){
    super(x,y,w,h);
    setBuffered(true);
    x = diam;
    rect = new Rect(0,0,diam,diam);
  }
  
  public void start(){
    thread = new Thread(this);
    thread.start();
  }
  
  public void run(){
    while(true){
      draw();
      x++;  // move the shape
      x = x%width();  // wrap it from right to left
    }
  }
  
  public void drawView(Graphics g){
    // draw a black background
    g.setColor(Color.black);
    g.fillRect(localBounds());
    // draw a rolling shape
    angle = -(360/diam)*x;
    rect.setBounds(x,(height()-diam)/2,diam,diam);
    g.setColor(Color.green);
    g.drawOval(rect);
    g.fillArc(rect,angle,90);
    g.fillArc(rect,angle+180,90);
  }
  
  public void stop(){
    thread.stop();
    thread = null;
  }
}

// a class that performs flicker-free animation by
// implementing its own off-screen drawing with Bitmap 
class OffScreenRoller extends View implements Runnable{
  Bitmap bitmap;
  Graphics onscreen,bitmapGraphics;
  Thread thread;
  int x,angle;
  Rect currentRect,oldRect;
  int diam = 30;
 
  public OffScreenRoller(int x,int y,int w,int h){
    super(x,y,w,h);
    x = 0;
    currentRect = new Rect(x,(height()-diam)/2,diam,diam);
    oldRect = new Rect(currentRect);
    bitmap = new Bitmap(w,h);
  }

  public void start(){
    onscreen = createGraphics();
    bitmapGraphics = bitmap.createGraphics();
    thread = new Thread(this);
    thread.start();
  }
  
  public void run(){
    while(true){
      // compute the current position of the roller
      currentRect.setBounds(x,(height()-diam)/2,diam,diam);
      
      // compute the clipping rectangle by taking the union
      // with the old position
      Rect clippingRect = Rect.rectFromUnion(currentRect,oldRect);

      // set the clipping rectangle and draw in the bitmap.
      // Note the second argument, false.  This clears any previous
      // clipping rectangles from bitmapGraphics.
      bitmapGraphics.setClipRect(clippingRect,false);
      drawView(bitmapGraphics);

      // set the clipping rectangle and draw in the View.
      onscreen.setClipRect(clippingRect,false);
      onscreen.drawBitmapAt(bitmap,0,0);

      x++;  // move the shape
      x = x%width();  // wrap it from right to left
      oldRect.setBounds(currentRect);
    }
  }
  
  public void drawView(Graphics g){
    // draw a black background
    g.setColor(Color.black);
    g.fillRect(localBounds());

    // draw a rolling shape
    angle = -(360/diam)*x;
    g.setColor(Color.green);
    g.drawOval(currentRect);
    g.fillArc(currentRect,angle,90);
    g.fillArc(currentRect,angle+180,90);
  }
  
  public void stop(){
    bitmapGraphics.dispose();  // don't forget to do this!
    onscreen.dispose();        // or this!
    thread.stop();
    thread = null;
  }
}
