import netscape.application.*;
import netscape.util.*;

// A ListView that can have ListItems dragged out of it
public class DragListView extends ListView 
             implements DragSource,DragDestination{
  public DragListView(Rect r){
    super(r);
    setAllowsMultipleSelection(false);
  }
  
  public void mouseDragged(MouseEvent e){
    if(selectedItem() == null) return;
    if(!rectForItem(selectedItem()).contains(e.x,e.y))
      createDragSession(selectedItem(),e.x,e.y);
  }

  protected void createDragSession(ListItem item,int x,int y){
    // create an Image based on the ListItem
    Bitmap bitmap = new Bitmap(item.minWidth(),item.minHeight());
    Graphics gr = bitmap.createGraphics();
    item.drawInRect(gr,new Rect(0,0,item.minWidth(),item.minHeight()));
    gr.dispose();
    // create a DragSession with the Bitmap and the ListItem
    DragSession dragSession = 
          new DragSession(this,bitmap,x,y,x,y,"ListItem",item);
  }

  // a View method related to drag and drop
  public DragDestination acceptsDrag(DragSession ds, int x, int y){
    return this;
  }

  // DragSource methods
  public void dragWasAccepted(DragSession ds){
    // if the ListItem was accepted by a DragDestination,
    // remove it from the list.
    removeItem((ListItem) ds.data());
    draw();
  }

  public boolean dragWasRejected(DragSession ds){
    return true;
  }

  public View sourceView(DragSession ds){
    return this;
  }

  // DragDestination methods
  public boolean dragDropped(DragSession ds){
    if(ds.source() == this) return false;  // reject if 'this' is the source
    // reject everything that isn't a ListItem
    if(!(ds.data() instanceof ListItem)) return false; 
    ListItem draggedItem = (ListItem) ds.data();
    ListItem newItem = (ListItem) draggedItem.clone();  // clone the dragged item
    addItem(newItem); // add the new ListItem
    selectItem(newItem);
    draw();
    return true;
  }

  public boolean dragEntered(DragSession ds){
    if(ds.source() == this) return false;  // reject if 'this' is the source
    else if(ds.data() instanceof ListItem) return true;  // accept all ListItems
    return false;  // reject everything else
  }

  public boolean dragMoved(DragSession ds){
    return dragEntered(ds);  
  }

  public void dragExited(DragSession ds){
    // no special behavior was implemented by dragEntered(),
    // so this method does nothing.
  }
}
